/* -*-C-*- os.h */
/*-->os*/
/**********************************************************************/
/********************************* os *********************************/
/**********************************************************************/

#ifndef OS_H_DEFINED__ /* file ignored if included more than once */
#define OS_H_DEFINED__

/***********************************************************************
This file sets the operating system and compiler flags, automatically
where this is possible based on pre-defined symbols, and also always
includes stdio.h (this has been found essential to resolve problems of
where to find open file limits).

	=================		=========
	Operating Systems		Compilers
	=================		=========
	OS_ATARI			(Mark Williams)
	OS_PCDOS			__TURBOC__ and (Microsoft C)
	OS_PRIMOS			Prime
	OS_RMX				Intel
	OS_TOPS20			KCC_20 and PCC_20
	OS_UNIX				(cc and GNU gcc)
	OS_UNIX and OS_VAXVMS		__GNUC__
	OS_VAXVMS			(VMS cc and GNU gcc)
	OS_VMCMS			(Waterloo)
	any				__STDC__ (ANSI conformant)
	=================		=========

The operating system macros will be given definitions (0 or 1), so
they can freely by used in later preprocessor expressions.  Specific
compiler symbols should be used only inside code bracketed by #if
OS_xxx ...  #endif.

For convenience, the following values are also set:

	STDC		0 in pre-ANSI C, 1 in ANSI (or almost ANSI) C

	UNIX_ATT	UNIX variants
	UNIX_BSD

	ARGS		argument prototypes (ANSI C or pre-ANSI C)
	const		type modifier (new with ANSI C)
	errno		UNIX style error code
	MAXPATHLEN	length of the longest host filename.
	NULL		null pointer constant
	OPENMAX		maximum number of open files per process
	fpos_t		type used for file positions
	offsetof	macro to return offset of element in structure
	ptrdiff_t	type used for pointer differences
	R_OPEN		fopen() mode flags for text read
	RB_OPEN		fopen() mode flags for binary read
	RW_OPEN		fopen() mode flags for text read/write
	size_t		integer type returned by sizeof()
	time_t		integer type used for time counters
	W_OPEN		fopen() mode flags for text write
	WB_OPEN		fopen() mode flags for binary write
	VOIDP		void* (new with ANSI C) or char* (pre-ANSI C)
	volatile	type modifier (new with ANSI C)

Some of these are defined in ANSI stddef.h; a private version of that
is included below, so it need not appear in user programs.

It does not appear possible for OS_PRIMOS, OS_RMX, and OS_VMCMS to
recognize the operating system automatically from predefined symbols;
for those, you will have to set the OS_xxx value explicitly below, or
else define it at compile time.

[07-Jul-1990]
***********************************************************************/

/***********************************************************************
ANSI C conformant systems must define __STDC__ to a non-zero value.
All predefined macros in ANSI C systems have leading and trailing
double underscores, and ANSI C reserves names of that form for the
compiler/library implementors.	Alas, prior practice has been to
predefine arbitrary names, usually in lower case, without underscores
(thereby making them likely to be confused with program variables).
We use some of these in tests below.  It is better to define our own
names (all in upper case) and make them more systematic and less
likely to clash with predefined symbols.

Alas, some compilers (Microsoft C 5.1, QuickC 2.0) predefine __STDC__
to a zero value that cannot be changed, and others (GNU gcc) define it
to a non-zero value when the library and system header files are not
yet conformant.	 Therefore, we only test if __STDC__ is defined or
not, and we use a private symbol, STDC, for deciding whether we have
an ANSI (or almost ANSI) C environment.	 We use this to choose the
form of function prototypes, and to decide whether certain #include
header files and predefined types are available.
***********************************************************************/

/* Here are the operating systems supported. */

#ifndef OS_ATARI
#define OS_ATARI		0
#endif /* OS_ATARI */

#ifndef OS_PCDOS
#define OS_PCDOS		0
#endif /* OS_PCDOS */

#ifndef OS_PRIMOS
#define OS_PRIMOS		0
#endif /* OS_PRIMOS */

#ifndef OS_RMX
#define OS_RMX			0
#endif /* OS_RMX */

#ifndef OS_TOPS20
#define OS_TOPS20		0
#endif /* OS_TOPS20 */

#ifndef OS_UNIX
#define OS_UNIX			0
#endif /* OS_UNIX */

#ifndef OS_VAXVMS
#define OS_VAXVMS		0
#endif /* OS_VAXVMS */

#ifndef OS_VMCMS
#define OS_VMCMS		0
#endif /* OS_VMCMS */

/* Provide defaults for compiler symbols */

#ifndef IBM_PC_LATTICE
#define IBM_PC_LATTICE		0
#endif /* IBM_PC_LATTICE */

#ifndef IBM_PC_MICROSOFT
#define IBM_PC_MICROSOFT	0
#endif /* IBM_PC_MICROSOFT */

#ifndef IBM_PC_TURBO
#define IBM_PC_TURBO		0
#endif /* IBM_PC_TURBO */

#ifndef KCC_20
#define KCC_20			0
#endif /* KCC_20 */

#ifndef PCC_20
#define PCC_20			0
#endif /* PCC_20 */

#ifndef UNIX_ATT
#define UNIX_ATT		0
#endif /* UNIX_ATT */

#ifndef UNIX_BSD
#define UNIX_BSD		0
#endif /* UNIX_BSD */

/***********************************************************************
Now check for the definition of certain predefined symbols to see if
we can automatically set the operating system and UNIX variant flags.
This is possible for OS_PCDOS, OS_TOPS20 (KCC_20 only), OS_VAXVMS, and
OS_UNIX, but not for OS_TOPS20 (PCC_20) or OS_ATARI.

On IBM VM/CMS, IBM C and Waterloo C have only 'ANSI' predefined if you
compile under that option, so there is no apparently no way to detect
that environment automatically.	 I have no information yet on what the
SAS/Lattice C compiler does.  Compile-time definition of OS_VMCMS will
therefore be necessary.
***********************************************************************/

#ifdef _AIX
/***********************************************************************
Both AIX-370 with the MetaWare High C Compiler R2.1n, and IBM AIX PS/2
C Language - Version 1.1, support ANSI-style function prototypes and
declarations, and have ANSI include files.  The preprocessor symbols
defined by these compilers are:

PS/2:		_AIX	AIX		NLS	ps2
3090:		_AIX	AIX	_AIX370	NLS	__HIGHC__	__STDC__
RS/6000:	_AIX	_IBMR2
***********************************************************************/

#ifdef OS_UNIX
#undef OS_UNIX
#endif /* OS_UNIX */

#ifndef _IBMR2
typedef long fpos_t;			/* should be defined, but is not */
#endif /* _IBMR2 */

#define OS_UNIX		1
#define STDC		1
#endif /* _AIX */

/* NB: We check for AIX separately from _AIX.  Both are defined by
ANSI-level compilers on AIX, but _AIX is not defined by pre-ANSI
compilers */

#ifdef AIX	/* IBM PS/2 and RT AIX (System V.2 UNIX + BSD extensions) */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* AIX */

#ifdef LATTICE	/* Lattice C 6.0 or later */

#ifdef OS_PCDOS
#undef OS_PCDOS
#endif /* OS_PCDOS */

#define OS_PCDOS		1

#ifdef IBM_PC_LATTICE
#undef IBM_PC_LATTICE
#endif /* IBM_PC_LATTICE */

#define IBM_PC_LATTICE		1
#define STDC			1
#define stdprn			stdprt	/* Lattice name different, sigh... */
#endif /* LATTICE */

#ifdef MSDOS  /* Microsoft C 3, 4, 5, Quick C 1, 2 (latter defines _QC too) */

#ifdef OS_PCDOS
#undef OS_PCDOS
#endif /* OS_PCDOS */

#define OS_PCDOS		1

#ifdef IBM_PC_MICROSOFT
#undef IBM_PC_MICROSOFT
#endif /* IBM_PC_MICROSOFT */

#define IBM_PC_MICROSOFT	1
#define STDC			1
#endif /* MSDOS */

#ifdef __TURBOC__ /* Turbo C 2.0 (it also defines __MSDOS__) */
		/* __STDC__ is predefined only if -A compile switch used */

#ifdef OS_PCDOS
#undef OS_PCDOS
#endif /* OS_PCDOS */

#define OS_PCDOS		1

#ifdef IBM_PC_TURBO
#undef IBM_PC_TURBO
#endif /* IBM_PC_TURBO */

#define IBM_PC_TURBO		1
#define STDC			1
#endif /* __TURBOC__ */

#ifdef __COMPILER_KCC__

#ifdef OS_TOPS20
#undef OS_TOPS20
#endif /* OS_TOPS20 */

#define OS_TOPS20		1

#ifdef KCC_20
#undef KCC_20
#endif /* KCC_20 */

#define KCC_20			1

#ifdef __STDC__
#define STDC			1	/* KCC 6 or later */
#endif

#endif /* __COMPILER_KCC__ */

#if    KCC_20

#ifdef OS_TOPS20
#undef OS_TOPS20
#endif /* OS_TOPS20 */

#define OS_TOPS20		1
#endif /* KCC_20 */

#if    PCC_20

#ifdef OS_TOPS20
#undef OS_TOPS20
#endif /* OS_TOPS20 */

#define OS_TOPS20		1
#endif /* PCC_20 */

#if    OS_TOPS20
#if    (KCC_20 | PCC_20)
#else  /* NOT (KCC_20 | PCC_20) */

#ifdef KCC_20
#undef KCC_20
#endif /* KCC_20 */

#define KCC_20			1	/* KCC-20 is default for Tops-20 */
#endif /* (KCC_20 | PCC_20) */
#endif /* OS_TOPS20 */

#ifdef VMS

#ifdef OS_VAXVMS
#undef OS_VAXVMS
#endif /* OS_VAXVMS */

#define OS_VAXVMS		1
#define STDC			1
#endif /* VMS */

#ifdef unix	/* (all?) UNIX systems define this */

#ifdef OS_UNIX
#undef OS_UNIX
#endif /* OS_UNIX */

#define OS_UNIX			1
#endif /* unix */

#ifdef ardent	/* Ardent Titan UNIX (System V + BSD extensions) */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* ardent */

#ifdef hpux	/* Hewlett-Packard UNIX (System V + BSD extensions) */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* hpux */

#ifdef mips	/* MIPS UNIX (System V.3) */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* mips */

#ifdef sgi	/* Silicon Graphics UNIX System V.x */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* sgi */

#ifdef __sgi	/* Silicon Graphics Iris Indigo */
#define STDC			1
#define HAVE_SSCANF_FUNCTION	1
#define HAVE_SPRINTF_DCL	1
#define HAVE_OPEN_DCL		1
#define HAVE_CREAT_DCL		1
#define HAVE_CLOSE_DCL		1
#define HAVE_STDDEF_H		1
#define HAVE_STDLIB_H		1
#endif /* __sgi */

#ifdef SVR0	/* Silicon Graphics UNIX System V.x */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* SVR0 */

#ifdef sun	/* Sun Microsystems UNIX OS 3.x and 4.x (4.2BSD) */
#ifdef sun386				/* Sun 386i */
#define HAVE_SSCANF_FUNCTION	1	/* true for all versions */
#define HAVE_SPRINTF_DCL	1	/* true at SunOS 4.0 or later */
#else					/* Sun 3 or 4 */
#define HAVE_SSCANF_FUNCTION	1	/* true for all versions */
#define HAVE_SPRINTF_DCL	1	/* true at SunOS 4.0 or later */
#define HAVE_OPEN_DCL		1	/* true at SunOS 4.1 or later */
#define HAVE_CREAT_DCL		1	/* true at SunOS 4.1 or later */
#define HAVE_CLOSE_DCL		1	/* true at SunOS 4.1 or later */

#define HAVE_STDDEF_H		1	/* true at SunOS 4.1 or later, even */
					/* though cc is not yet Standard C */
#define HAVE_STDLIB_H		1	/* true at SunOS 4.1 or later, even */
					/* though cc is not yet Standard C */
#endif

#ifdef UNIX_BSD
#undef UNIX_BSD
#endif /* UNIX_BSD */

#define UNIX_BSD		1
#endif /* sun */

#ifdef ultrix	/* DEC version of 4.2BSD UNIX */

#ifdef UNIX_BSD
#undef UNIX_BSD
#endif /* UNIX_BSD */

#define UNIX_BSD		1

#ifdef UNIX_ATT	/* DECstation 3100 defines mips but is ULTRIX, not System V */
#undef UNIX_ATT
#endif
#endif /* ultrix */

#ifdef unixpc	/* UNIX PC (System V) */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* unixpc */

#ifdef usg	/* AT&T UNIXes may be define this (GNU EMACS uses USG) */

#ifdef UNIX_ATT
#undef UNIX_ATT
#endif /* UNIX_ATT */

#define UNIX_ATT		1
#endif /* usg */

#ifdef NeXT
/* NeXT systems use the GNU C compiler and MACH (4.3BSD system call
compatible).  The compiler predefines __MACH__, mc68000, NeXT,
__STDC__, and unix, and has ANSI-conformant library and header files. */
#define STDC			1

#ifdef UNIX_BSD
#undef UNIX_BSD
#endif /* UNIX_BSD */

#define UNIX_BSD		1
#endif /* NeXT */

#if   (OS_ATARI | OS_PCDOS | OS_PRIMOS | OS_RMX | OS_TOPS20)
#else /* NOT (OS_ATARI | OS_PCDOS | OS_PRIMOS | OS_RMX | OS_TOPS20) */
#if   (OS_UNIX | OS_VAXVMS | OS_VMCMS)
#else /* NOT (OS_UNIX | OS_VAXVMS | OS_VMCMS) */
			/* OS_UNIX is default if no operating system defined */

#ifdef OS_UNIX
#undef OS_UNIX
#endif /* OS_UNIX */

#define OS_UNIX			1
#endif /* (OS_UNIX | OS_VAXVMS | OS_VMCMS) */
#endif /* (OS_ATARI | OS_PCDOS | OS_PRIMOS | OS_RMX | OS_TOPS20) */

#if    OS_UNIX
#if    (UNIX_ATT | UNIX_BSD)
#else  /* NOT (UNIX_ATT | UNIX_BSD) */

#ifdef UNIX_BSD
#undef UNIX_BSD
#endif /* UNIX_BSD */

#define UNIX_BSD		1	/* UNIX_BSD is default UNIX variant */
#endif /* (UNIX_ATT | UNIX_BSD) */
#endif /* OS_UNIX */

#ifndef STDC
#define STDC			0
#endif /* STDC */

#include "xtypes.h"			/* private version protects against */
					/* multiple includes, and handles */
					/* varying locations of types.h */

#if    STDC
#define ARGS(parenthesized_list) parenthesized_list
#define VOIDP			void*
#else  /* NOT STDC */
/* Provide some synonyms for new ANSI C features */
#define ARGS(parenthesized_list) ()
#define VOIDP			char*

/* A few systems know about const, even though they are not yet STDC.  For
the rest, make the type an empty string. */
#ifndef __GNUC__
#ifndef _AIX
#ifndef VMS
#ifndef HAVE_CONST_TYPE
#define const
#endif /* HAVE_CONST */
#endif /* VMS */
#endif /* _AIX */
#endif /* __GNUC__ */

#ifndef volatile
#define volatile
#endif /* volatile */

#ifndef fpos_t
#ifndef sgi
#ifndef hpux				/* HP-UX uses typedef for this */
#ifndef ultrix
#define fpos_t			unsigned long
#endif /* ultrix */
#endif /* hpux */
#endif /* sgi */
#endif /* fpos_t */

#ifndef size_t
#ifndef sgi
#ifndef hpux				/* HP-UX uses typedef for this */
#ifndef ultrix
#define size_t			unsigned int
#endif /* ultrix */
#endif /* hpux */
#endif /* sgi */
#endif /* size_t */

#ifndef time_t
#ifndef hpux				/* HP-UX uses typedef for this */
#ifndef ultrix
#define time_t			long
#endif /* ultrix */
#endif /* hpux */
#endif /* time_t */

#endif /* STDC */

#ifdef ardent			/* Ardent cc supports ANSI prototypes */
#undef ARGS
#define ARGS(parenthesized_list) parenthesized_list
#endif /* ardent */

#ifdef _LANGUAGE_C
/***********************************************************************
Silicon Graphics MIPS-based IRIS systems support ANSI-style prototypes,
but do not have the header files new with ANSI C.  The compiler
predefines these symbols:

	LANGUAGE_C		MIPSEB			SVR3
	SYSTYPE_SYSV		_LANGUAGE_C		_MIPSEB
	_SVR3			_SYSTYPE_SYSV		host_mips
	mips			sgi			unix

We cannot use sgi here for the test, because that is also defined by
older SGI systems that do not support prototypes.  We choose _LANGUAGE_C
since it seems to be unusual, and may characterize this system uniquely.

Unfortunately, the compiler version labelled Mips Computer Systems
Release 1.31 has a bug that prevents compilation of function prototypes
containing variable names as well as types.  When they fix it, we can
make ARGS do something useful, and the next two lines can then be moved
outside this comment.
#undef ARGS
#define ARGS(parenthesized_list) parenthesized_list
***********************************************************************/
#endif /* _LANGUAGE_C */

#ifdef __GNUC__
#undef ARGS
#define ARGS(parenthesized_list) parenthesized_list
#if    STDC
#else  /* NOT STDC */
#ifndef fpos_t				/* need this for stdlib.h */
#define fpos_t			unsigned long
#endif /* fpos_t */
#endif /* STDC */
#endif /* __GNUC__ */

/***********************************************************************
The system limit on the number of open files usually is set in a system
include file, but regrettably has no standard name.  The following have
been observed:

--------	-------------	------------------------------------------
Name		Definition	System
--------	-------------	------------------------------------------
IOFILES		<stdio.h>	Wizard C
NFILE		<stdio.h>	V7 UNIX
NOFILE		<sys/param.h>	Sun OS (4.2BSD), 4.3BSD and Gould UTX/32
OPEN_MAX	<limits.h>	POSIX
OPEN_MAX	<stdio.h>	Turbo C 2.0
SYS_OPEN	<stdio.h>	KCC-20, Turbo C 2.0, Microsoft C 5.0,
				Wizard C
_NFILE		<stdio.h>	VAX VMS, NMTCC-20, PCC-20, SYS V, HPUX,
				Microsoft C 5.0, Lattice C, Primos,
				IBM AIX (370, PS/2, RT)
_NFILE		<sys/param.h>	KCC-20
FOPEN_MAX	<stdio.h>	Intel RMX, IBM AIX (RS)
--------	-------------	------------------------------------------

We use a private symbol OPENMAX;  it is needed for cached-font support
in gblvars.h.
***********************************************************************/

/* Include stdio.h, and if available, param.h, and maybe, limits.h */

#include <stdio.h>

#if    OS_VAXVMS
			/* no param.h on VAX VMS */
#else  /* NOT OS_VAXVMS */
#ifdef NO_SYS_DIR			/* from xtypes.h */
#include <param.h>
#undef NO_SYS_DIR
#else /* NOT NO_SYS_DIR */
#include <sys/param.h>
#endif /* NO_SYS_DIR */
#endif /* OS_VAXVMS */

#ifndef OPENMAX
#ifdef FOPEN_MAX
#define OPENMAX			FOPEN_MAX
#endif /* FOPEN_MAX */
#endif /* OPENMAX */

#ifndef OPENMAX
#ifdef NFILE
#define OPENMAX			NFILE
#endif /* NFILE */
#endif /* OPENMAX */

#ifndef OPENMAX
#ifdef _NFILE
#define OPENMAX			_NFILE
#endif /* _NFILE */
#endif /* OPENMAX */

#ifndef OPENMAX
#ifdef NOFILE
#define OPENMAX			NOFILE
#endif /* NOFILE */
#endif /* OPENMAX */

#ifndef OPENMAX
#ifdef SYS_OPEN
#define OPENMAX			SYS_OPEN
#endif /* SYS_OPEN */
#endif /* OPENMAX */

#ifndef OPENMAX
#include <limits.h>
#ifdef OPEN_MAX
#define OPENMAX			OPEN_MAX
#endif /* OPEN_MAX */
#endif /* OPENMAX */

#ifndef OPENMAX
/* Cause a compilation error */
??? cannot find needed definition of OPENMAX ???
#endif /* OPENMAX */

/***********************************************************************
The limit on file name lengths is similarly non-standard.

--------	-------------	------------------------------------------
Name		Definition	System
--------	-------------	------------------------------------------
FNMAX		<stdio.h>	PCC-20
MAXPATH		<dir.h>		Turbo C 2.0
_MAX_PATH	<stdlib.h>	Microsoft C 5.0
MAXPATHLEN	<sys/param.h>	Sun OS (4.2BSD), 4.3BSD, Gould UTX/32,
				HPUX, KCC-20, AIX (RT, RS, PS/2, 370),
				HP/Apollo DomainOS
PATH_MAX	<stdio.h>	SYS V (Silicon Graphics)
PATH_MAX	<limits.h>	POSIX
FILENAME_MAX	<stdio.h>	Intel RMX, NeXT MACH
--------	-------------	------------------------------------------

We already have included <sys/param.h> or <limits.h>, but we don't want
to include <stdio.h>.  If we cannot find preset values, we set values
based on experience; this parameter is less critical than OPENMAX.
***********************************************************************/

#ifndef MAXPATHLEN
#ifdef PATH_MAX
#define MAXPATHLEN		PATH_MAX
#endif /* PATH_MAX */

#ifdef FNMAX

#ifdef MAXPATHLEN
#undef MAXPATHLEN
#endif /* MAXPATHLEN */

#define MAXPATHLEN		FNMAX
#endif /* FNMAX */

#ifdef FILENAME_MAX

#ifdef MAXPATHLEN
#undef MAXPATHLEN
#endif /* MAXPATHLEN */

#define MAXPATHLEN		FILENAME_MAX
#endif /* FILENAME_MAX */

#endif /* MAXPATHLEN */

#ifndef MAXPATHLEN

#if    OS_ATARI
#define MAXPATHLEN		128
#endif /* OS_ATARI */

#if    OS_PCDOS
#define MAXPATHLEN		80
#endif /* OS_PCDOS */

#if    OS_PRIMOS
#define MAXPATHLEN		128 /* not in system .h files */
#endif /* OS_PRIMOS */

#if    OS_RMX
#define MAXPATHLEN		64
#endif /* OS_RMX */

#if    OS_TOPS20
#define MAXPATHLEN		288
#endif /* OS_TOPS20 */

#if    OS_UNIX
#define MAXPATHLEN		1024	/* from <sys/param.h> in 4.3BSD */
#endif /* OS_UNIX */

#if    OS_VAXVMS
#define MAXPATHLEN		512	/* value uncertain */
#endif /* OS_VAXVMS */

#if    OS_VMCMS
#define MAXPATHLEN		128	/* value uncertain */
#endif /* OS_VMCMS */

#endif /* MAXPATHLEN */

/***********************************************************************
fopen() mode flags differ between systems.  Here are symbolic values:
	R_OPEN		-- text read
	RB_OPEN		-- binary read
	RW_OPEN		-- text read/write
	W_OPEN		-- text write
	WB_OPEN		-- binary write
***********************************************************************/

#define R_OPEN			"r"
#define RB_OPEN			"r"
#define RW_OPEN			"w+"
#define W_OPEN			"w"
#define WB_OPEN			"w"

#if    OS_PCDOS
#undef RB_OPEN
#define RB_OPEN			"rb"

#undef WB_OPEN
#define WB_OPEN			"wb"
#endif /* OS_PCDOS */

#if    OS_PRIMOS
#undef RB_OPEN
#define RB_OPEN			"i"

#undef RW_OPEN
#define RW_OPEN			"w"

#undef WB_OPEN
#define WB_OPEN			"o"
#endif /* OS_PRIMOS */

#if    OS_TOPS20
#undef RB_OPEN
#define RB_OPEN			"rb"

#undef WB_OPEN
#define WB_OPEN			"wb"
#endif /* OS_TOPS20 */

#if    OS_VAXVMS
#undef RB_OPEN
#define RB_OPEN			"rb"

#undef WB_OPEN
#define WB_OPEN			"wb"
#endif /* OS_VAXVMS */

#include "xstddef.h"

#if    (OS_PRIMOS | OS_VMCMS)
#else  /* NOT (OS_PRIMOS | OS_VMCMS) */
#include <errno.h>
#endif /* (OS_PRIMOS | OS_VMCMS) */

#if    (OS_ATARI | UNIX_BSD)
extern int errno;		/* not in all errno.h files, sigh... */
#endif /* (OS_ATARI | UNIX_BSD) */

#endif /* OS_H_DEFINED__ */
